﻿using ProjectAtelier.Reports;
using ProjectAtelier.Repositories;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Unity;

namespace ProjectAtelier.Forms
{
    public partial class FormMaterialReport : Form
    {
        private readonly IUnityContainer _container;

        public FormMaterialReport(IUnityContainer container, IMaterialRepository materialRepository)
        {
            InitializeComponent();
            _container = container;

            comboBoxMaterial.DataSource = materialRepository.ReadMaterials();
            comboBoxMaterial.DisplayMember = "Name";
            comboBoxMaterial.ValueMember = "Id";
        }

        private void buttonSelectFilePath_Click(object sender, EventArgs e)
        {
            var sfd = new SaveFileDialog()
            {
                Filter = "Excel Files | *.xlsx"
            };
            if (sfd.ShowDialog() != DialogResult.OK)
            {
                return;
            }
            textBoxFilePath.Text = sfd.FileName;
        }

        private void buttonMakeReport_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(textBoxFilePath.Text))
                {
                    throw new Exception("Отсутствует имя файла для отчета");
                }
                if (comboBoxMaterial.SelectedIndex < 0)
                {
                    throw new Exception("Не выбран корм");
                }
                if (dateTimePickerEnd.Value <= dateTimePickerStart.Value)
                {
                    throw new Exception("Дата начала должна быть раньше даты окончания");
                }
                if (_container.Resolve<TableReport>().CreateTable(textBoxFilePath.Text, (int)comboBoxMaterial.SelectedValue!,
                                                                    dateTimePickerStart.Value, dateTimePickerEnd.Value))
                {
                    MessageBox.Show("Документ сформирован", "Формирование документа", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show("Возникли ошибки при формировании документа. Подробности в логах", "Формирование документа",
                                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при создании очета",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        
    }
}

