﻿using Microsoft.Extensions.Logging;
using ProjectAtelier.REPOSITORY;

namespace ProjectAtelier.Reports;

class ChartReport
{
    private readonly IMaterialReplenishmentRepository _materialReplenishmentRepository;
    private readonly ILogger<ChartReport> _logger;

    public ChartReport(IMaterialReplenishmentRepository materialReplenishmentRepository, ILogger<ChartReport> logger)
    {
        _materialReplenishmentRepository = materialReplenishmentRepository ?? throw new ArgumentNullException(nameof(materialReplenishmentRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }

    public bool CreateChart(string filePath, DateTime dateTime)
    {
        try
        {
            new PdfBuilder(filePath)
                .AddHeader("Пополенение материала")
                .AddPieChart("Виды материалов", GetData(dateTime))
                .Build();
            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }
    private List<(string Caption, double Value)> GetData(DateTime dateTime)
    {
        return _materialReplenishmentRepository
            .ReadMaterialsSpent()
            .Where(x => x.DataTime.Date == dateTime.Date)
            .GroupBy(x => x.Name, (key, group) => new { Id = key, Count = group.Sum(x => x.Count) })
            .Select(x => (x.Id.ToString(), (double)x.Count))
            .ToList();
    }
}
