﻿using Microsoft.Extensions.Logging;
using ProjectAtelier.Repositories;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAtelier.Reports;

internal class DocReport
{
    private readonly IProductRepository _productRepository;
    private readonly IMaterialRepository _materialRepository;
    private readonly IClientRepository  _clientRepository;

    private readonly ILogger<DocReport> _logger;
   

    public DocReport(IProductRepository productRepository, IMaterialRepository materialRepository, IClientRepository clientRepository, ILogger<DocReport> logger)
    {
        _productRepository = productRepository ?? throw new ArgumentNullException(nameof(productRepository));
        _materialRepository = materialRepository ?? throw new ArgumentNullException(nameof(materialRepository));
        _clientRepository= clientRepository ?? throw new ArgumentNullException(nameof(clientRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }

    public bool CreateDoc(string filePath, bool includeProducts, bool includeMaterials, bool includeClients)
    {
        try
        {
            var builder = new WordBuilder(filePath).AddHeader("Документ со справочниками");

            if (includeProducts)
            {
                builder.AddParagraph("Изделия").AddTable([2400, 2400, 2400], GetProducts());
            }
            if (includeMaterials)
            {
                builder.AddParagraph("Материалы").AddTable([2400, 2400, 2400], GetMaterials());
            }
            if (includeClients)
            {
                builder.AddParagraph("Клиенты").AddTable([2400, 2400, 2400], GetClients());
            }
            builder.Build();
            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }
    private List<string[]> GetProducts()
    {
        return [
            ["Название", "Вид", "Количество материала"],
            .. _productRepository
                .ReadProducts()
                .Select(x => new string[] { x.Name, x.View.ToString(), x.CountMaterial.ToString() }),
        ];
    }

    private List<string[]> GetMaterials()
    {
        return [
            ["Название", "Используемое количество ", "Количество на складе"],
            .. _materialRepository
                .ReadMaterials()
                .Select(x => new string[] { x.Name, x.UseCount.ToString(), x.CountInWareHouse.ToString() }),
        ];
    }
    private List<string[]> GetClients()
    {
        return [
            ["Название", "Телефон", "Гендер"],
            .. _clientRepository
                .ReadClients()
                .Select(x => new string[] { x.Name, x.Phone.ToString(), x.Gender.ToString() }),
        ];
    }
}
