﻿using Microsoft.Extensions.Logging;
using ProjectAtelier.Repositories;
using ProjectAtelier.REPOSITORY;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAtelier.Reports;

internal class TableReport
{
    private readonly IMaterialReplenishmentRepository _materialReplenishmentRepository;
    private readonly IMaterialRepository _materialRepository;
    private readonly IOrderMaterialsRepository _orderMaterialsRepository;
    private readonly ILogger<TableReport> _logger;
    internal static readonly string[] item = ["Дата", "Количество пришло", "Количество ушло"];

    public TableReport(IMaterialReplenishmentRepository materialrepRepository,
                        IMaterialRepository materialRepository, IOrderMaterialsRepository orderMaterialsRepository, ILogger<TableReport> logger)
    {
        _materialReplenishmentRepository = materialrepRepository ?? throw new ArgumentNullException(nameof(materialrepRepository));
        _materialRepository = materialRepository ?? throw new ArgumentNullException(nameof(materialRepository));
        _orderMaterialsRepository = orderMaterialsRepository ?? throw new ArgumentNullException(nameof(materialRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));
    }

    public bool CreateTable(string filePath, int materialId, DateTime startDate, DateTime endDate)
    {
        try
        {
            new ExcelBuilder(filePath)
                .AddHeader("Сводка по движению материала", 0, 3)
                .AddParagraph("за период", 0)
                .AddTable([10, 15, 15], GetData(materialId, startDate, endDate))
                .Build();
            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }

    private List<string[]> GetData(int materialId, DateTime startDate, DateTime endDate)
    {
        var data = _materialReplenishmentRepository
            .ReadMaterialsSpent()
           .Where(x => x.DataTime >= startDate && x.DataTime <= endDate && x.Name == _materialRepository.ReadMaterialById(materialId).Name)
            .Select(x => new { Date = x.DataTime, CountIn = (int?)x.Count, CountOut = (int?)null })
            .Union(
                _orderMaterialsRepository
                    .ReadOrders(materialId)
                    .Where(x => x.DataOrder >= startDate && x.DataOrder <= endDate)
                    .Select(x => new { Date = x.DataOrder, CountIn = (int?)null, CountOut = (int?)x.Count }))
            .OrderBy(x => x.Date);
        return new List<string[]>() { item }
                .Union(
                    data
                        .Select(x => new string[] { x.Date.ToString(), x.CountIn?.ToString() ?? string.Empty, x.CountOut?.ToString() ?? string.Empty }))
                .Union(
                    [["Всего", data.Sum(x => x.CountIn ?? 0).ToString(), data.Sum(x => x.CountOut ?? 0).ToString()]])
                .ToList();
    }
}
