﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectAtelier.Entities;

namespace ProjectAtelier.Repositories.Implementations;

public class OrderMaterialsRepository : IOrderMaterialsRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<ProductRepository> _logger;

    public OrderMaterialsRepository(IConnectionString connectionString, ILogger<ProductRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }

    public IEnumerable<TempOrder> ReadOrders(int matid)
    {
        using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
        var querySelect = @"
            SELECT
orders.datatime AS DataOrder,
orders.*,  materials.id AS materialid, pm.count*op.count AS count
            FROM orders
            INNER JOIN order_products AS op ON orders.id = op.orderid
            INNER JOIN products ON op.productid = products.id
            INNER JOIN product_materials AS pm ON products.id = pm.productid
            INNER JOIN materials ON pm.materialid = materials.id
            Where materials.id = @matid";
        var orderMaterials = connection.Query<TempOrder>(querySelect, new { matid });
        _logger.LogDebug("Полученные объекты: {json}", JsonConvert.SerializeObject(orderMaterials));
        return orderMaterials;
    }
}
